//
//  OCRPerformer.swift
//  scanbot_sdk
//
//  Created by Zhenya Knizhnik on 02.09.19.
//

import Foundation
import ScanbotSDK
import Flutter

class OCRPerformer {
    func perform(with config: OCRPerformConfiguration,
                 result: @escaping FlutterResult) {
        
        let storage = SBSDKIndexedImageStorage.temporaryStorage(for: config.pages)
        var outputPDFURL: URL?
        if config.shouldGeneratePdf {
            outputPDFURL = PluginPreferences.shared.newStorageCustomFileURL(extensionString: "pdf")
        }
        
        let configuration: SBSDKOpticalCharacterRecognizerConfiguration
        if config.engine == .ML || config.languages.isEmpty {
            configuration = SBSDKOpticalCharacterRecognizerConfiguration.ml()
        } else {
            configuration = SBSDKOpticalCharacterRecognizerConfiguration.legacyConfiguration(withLanguages: config.languages)
        }
        SBSDKOpticalCharacterRecognizer(configuration: configuration).recognizeText(storage) { ocrResult, error in
            if let error {
                result(OCRPerformerError.generic(message: error.localizedDescription))
                storage.removeAllImages()
            } else {
                if let ocrResult {
                    if config.shouldGeneratePdf {
                        Renderer().renderPDF(from: config.pages,
                                             pageSize: config.pdfOptions.pageSize,
                                             pageOrientation: config.pdfOptions.pageOrientation, 
                                             pdfOutputURL: outputPDFURL) { outputURL in
                            if let pdfAttributes = config.pdfAttributes {
                                SBSDKPDFMetadataProcessor.writeMetadata(pdfAttributes.metadata,
                                                                        withPDFFrom: outputURL,
                                                                        to: outputURL,
                                                                        removeCurrentMeta: false)
                            }
                            DispatchQueue.main.async {
                                let resultOCR = OCRResult(sdkORCResult: ocrResult, pdfURL: outputURL)
                                result(resultOCR.encodedString)
                            }
                          
                        }
                    } else {
                        let resultOCR = OCRResult(sdkORCResult: ocrResult, pdfURL: nil)
                        result(resultOCR.encodedString)
                        storage.removeAllImages()
                    }
                } else {
                    result(OCRPerformerError.unknown(for: .ocr).value)
                    storage.removeAllImages()
                }
            }
        }
    }
}
