//
//  CroppingNativeViewConfiguration.swift
//  scanbot_sdk
//
//  Created by Seifeddine Bouzid on 28.03.23.
//

import Foundation
import ScanbotSDK

struct CroppingNativeViewConfiguration {
    
    var edgeLineWidth: Double?
    var edgeColor: UIColor?
    var anchorPointsColor: UIColor?
    var edgeColorOnLine: UIColor?
    var borderInsets: UIEdgeInsets?
    var page: SBSDKUIPage?
    
    init() { }
    
    init(_ arguments: [String: Any?]) {
        self = CroppingNativeViewConfiguration.parseArguments(arguments: arguments)
    }
    
    private static func parseArguments(arguments: [String: Any?]) -> CroppingNativeViewConfiguration {
        var configuration = CroppingNativeViewConfiguration()
        
        if let edgeLineWidth =  arguments["edgeLineWidth"] as? Double {
            configuration.edgeLineWidth = edgeLineWidth
        }
        if let hex = arguments["edgeColor"] as? String {
            configuration.edgeColor = UIColor(hexString: hex)
        }
        if let hex = arguments["anchorPointsColor"] as? String {
            configuration.anchorPointsColor = UIColor(hexString: hex)
        }
        if let hex = arguments["edgeColorOnLine"] as? String {
            configuration.edgeColorOnLine = UIColor(hexString: hex)
        }
        if let insets = arguments["borderInsets"] as? [String: Any] {
            if let insets = UIEdgeInsets(from: insets) {
                configuration.borderInsets = insets
            }
        }
        if let page = self.parsePage(arguments: arguments) {
            configuration.page = page
        }
        return configuration
    }
    
    private static func parsePage(arguments: [String: Any?]) -> SBSDKUIPage? {
        let pageDictionary = arguments["page"] as? [String: Any?] ?? arguments
        guard let id = pageDictionary["pageId"] as? String, let uuid = UUID(uuidString: id) else {
            return nil
        }
        
        let polygon = SBSDKPolygon(from: pageDictionary)
        if let detectionStatus = pageDictionary["detectionStatus"] as? String {
            polygon?.detectionStatus = SBSDKDocumentDetectionStatus(statusString: detectionStatus)
        }
        
        var filter = SBSDKImageFilterTypeNone
        if let filterString = pageDictionary["filter"] as? String {
            filter = SBSDKImageFilterType(string: filterString)
        }
        
        var size = CGSize.zero
        if let sizeDictionary = pageDictionary["documentImageSizeLimit"] as? [String: Any] {
            size = CGSize(from: sizeDictionary)
        }
        return SBSDKUIPage(pageFileID: uuid, polygon: polygon, filter: filter, documentImageSizeLimit: size)
    }
}
