import 'package:json_annotation/json_annotation.dart';

import 'camera_configuration.dart';

export 'camera_configuration.dart';
export 'finder.dart';
part 'medical_certificate_scanner_configuration.g.dart';

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class MedicalCertificateCameraConfiguration extends CameraConfiguration {
  /// Controls whether the flash should be initially enabled.
  /// The default value is FALSE.
  bool? flashEnabled;

  /// Set current zoom by a linear zoom value ranging from 0f to 1.0f. It maps to the range of real values of 1x-12x zoom.
  /// If device os not supporting some part of this range the closest value will be taken.
  /// Optical Zoom Level 0f represents the minimum zoom(1x) while Optical Zoom Level 1.0f represents the maximum zoom(12x).
  /// Default value is 0f.
  double? cameraZoomFactor;

  @override
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.mc;

  FinderConfiguration? finder;

  MedicalCertificateClassicScannerConfiguration scannerConfiguration;

  MedicalCertificateCameraConfiguration({
    required this.scannerConfiguration,
    this.flashEnabled,
    this.finder,
    this.cameraZoomFactor,
  });

  @override
  Map<String, dynamic> toJson() =>
      _$MedicalCertificateCameraConfigurationToJson(this);
}

@JsonSerializable(
    createFactory: false, includeIfNull: false, explicitToJson: true)
class MedicalCertificateClassicScannerConfiguration {
  @JsonKey(name: "detectorType")
  final DetectorType detectorType = DetectorType.mc;

  /// Recognition of Patient Information block. true if enabled
  bool recognizePatientInfo;

  /// Reading data from barcode, if it's present on document, could make data recognition more accurate. true if enabled.
  bool recognizeBarcode;

  /// Make high-res snapshot of the image after first successful medical certificate detection and recognise medical certificate fields on it
  bool captureHighResolutionImage;

  MedicalCertificateClassicScannerConfiguration(
      {this.recognizePatientInfo = true,
      this.recognizeBarcode = true,
      this.captureHighResolutionImage = false});

  Map<String, dynamic> toJson() =>
      _$MedicalCertificateClassicScannerConfigurationToJson(this);
}
