import 'package:json_annotation/json_annotation.dart';
import 'json/document_configuration_json.dart';

import 'json/common.dart';
export 'json/document_configuration_json.dart';

part 'document_scan_data.g.dart';

@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class DocumentContourScanningResult {
  List<PolygonPoint>? polygon;
  DetectionStatus detectionStatus;

  DocumentContourScanningResult(this.polygon, this.detectionStatus);

  factory DocumentContourScanningResult.fromJson(Map<String, dynamic> json) =>
      _$DocumentContourScanningResultFromJson(json);
}

/// Scanned document return class from document scanner rtu-ui component
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class DocumentScanningResult extends Result {
  List<Page> pages = <Page>[];

  DocumentScanningResult(this.pages);

  DocumentScanningResult.error(String message) {
    operationResult = OperationResult.ERROR;
    this.message = message;
  }

  DocumentScanningResult.canceled() {
    operationResult = OperationResult.CANCELED;
  }

  factory DocumentScanningResult.fromJson(Map<String, dynamic> json) =>
      _$DocumentScanningResultFromJson(json);

  Map<String, dynamic> toJson() => _$DocumentScanningResultToJson(this);
}

/// Document scanner configuration class. All UI configuration options are optional.
class DocumentScannerConfiguration extends DocumentScannerJsonConfiguration {
  /// The configuration object should be applied for this screen
  @override
  String? screen = "DocumentScanner";

  DocumentScannerConfiguration({
    super.acceptedAngleScore,
    super.acceptedSizeScore,
    super.autoSnappingButtonHidden,
    super.autoSnappingButtonTitle,
    super.autoSnappingEnabled,
    super.autoSnappingSensitivity,
    super.autoSnappingDelay,
    super.bottomBarBackgroundColor,
    super.bottomBarButtonsColor,
    super.cameraModule,
    super.cameraBackgroundColor,
    super.cameraPreviewMode,
    super.cameraUnavailableExplanationText,
    super.cancelButtonHidden,
    super.cancelButtonTitle,
    super.defaultPageFilter,
    super.detectorMode,
    super.enableCameraButtonTitle,
    super.enableCameraExplanationText,
    super.flashButtonHidden,
    super.flashButtonTitle,
    super.flashEnabled,
    super.forceUserGuidance,
    super.ignoreBadAspectRatio,
    super.imageScale,
    super.multiPageButtonHidden,
    super.multiPageButtonTitle,
    super.multiPageEnabled,
    super.orientationLockMode,
    super.pageCounterButtonTitle,
    super.photoQualityPrioritization,
    super.polygonBackgroundColor,
    super.polygonBackgroundColorOK,
    super.polygonColor,
    super.polygonColorOK,
    super.polygonLineWidth,
    super.polygonCornerRadius,
    super.polygonAutoSnapProgressColor,
    super.polygonAutoSnapProgressLineWidth,
    super.polygonAutoSnapProgressEnabled,
    super.requiredAspectRatios,
    super.preferredAspectRatios,
    super.shutterButtonAutoInnerColor,
    super.shutterButtonAutoOuterColor,
    super.shutterButtonIndicatorColor,
    super.shutterButtonManualInnerColor,
    super.shutterButtonManualOuterColor,
    super.stopsCameraSessionWhenDisappeared,
    super.textHintBadAngles,
    super.textHintBadAspectRatio,
    super.textHintNothingDetected,
    super.textHintOffCenter,
    super.textHintOK,
    super.textHintTooDark,
    super.textHintTooNoisy,
    super.textHintTooSmall,
    super.topBarBackgroundColor,
    super.topBarButtonsActiveColor,
    super.topBarButtonsInactiveColor,
    super.userGuidanceBackgroundColor,
    super.userGuidanceTextColor,
    super.userGuidanceFontSize,
    super.documentImageSizeLimit,
    super.shutterButtonHidden,
    super.textHintEnergySavingActive,
    super.maxNumberOfPages,
    super.useButtonsAllCaps,
    super.accessibilityConfiguration,
    super.touchToFocusEnabled,
  });
}
