import 'package:json_annotation/json_annotation.dart';
import 'json/common.dart';
import 'json/ehic_configuration_json.dart';

export 'json/ehic_configuration_json.dart';
part 'ehic_scanning_data.g.dart';

/// This class describes health insurance rtu-ui component scanned result
@JsonSerializable(
    createFactory: true, includeIfNull: false, explicitToJson: true)
class HealthInsuranceCardRecognitionResult extends Result {
  List<HealthInsuranceCardField> fields = <HealthInsuranceCardField>[];
  HealthInsuranceCardDetectionStatus? status;

  HealthInsuranceCardRecognitionResult(this.fields, this.status);

  factory HealthInsuranceCardRecognitionResult.fromJson(
          Map<String, dynamic> json) =>
      _$HealthInsuranceCardRecognitionResultFromJson(json);

  Map<String, dynamic> toJson() =>
      _$HealthInsuranceCardRecognitionResultToJson(this);

  HealthInsuranceCardRecognitionResult.canceled() {
    operationResult = OperationResult.CANCELED;
  }
}

/// Class that describes health insurance one scanned field
@JsonSerializable(includeIfNull: false, explicitToJson: true)
class HealthInsuranceCardField {
  final HealthInsuranceCardFieldType type;
  final String value;
  final double confidence;

  HealthInsuranceCardField(this.type, this.value, this.confidence);

  factory HealthInsuranceCardField.fromJson(Map<String, dynamic> json) =>
      _$HealthInsuranceCardFieldFromJson(json);

  Map<String, dynamic> toJson() => _$HealthInsuranceCardFieldToJson(this);
}

enum HealthInsuranceCardFieldType {
  SURNAME,
  GIVEN_NAME,
  DATE_OF_BIRTH,
  PERSONAL_IDENTIFICATION_NUMBER,
  INSTITUTION_NUMBER,
  INSTITUTION_NAME,
  CARD_NUMBER,
  CARD_EXPIRATION_DATE,
  COUNTRY
}

enum HealthInsuranceCardDetectionStatus {
  SUCCESS,
  FAILED_DETECTION,
  INCOMPLETE_VALIDATION
}

/// Health insurance rtu-ui scanner configuration class
class HealthInsuranceScannerConfiguration
    extends HealthInsuranceCardScannerJsonConfiguration {
  HealthInsuranceScannerConfiguration(
      {super.cameraModule,
      super.cameraOverlayColor,
      super.cancelButtonTitle,
      super.cancelButtonHidden,
      super.detectionStatusFailedDetectionText,
      super.detectionStatusFailedValidationText,
      super.detectionStatusSuccessText,
      super.enableCameraButtonTitle,
      super.enableCameraExplanationText,
      super.finderLineColor,
      super.finderLineWidth,
      super.finderTextHint,
      super.finderTextHintColor,
      super.flashButtonHidden,
      super.flashButtonTitle,
      super.flashEnabled,
      super.orientationLockMode,
      super.topBarBackgroundColor,
      super.topBarButtonsActiveColor,
      super.topBarButtonsInactiveColor,
      super.useButtonsAllCaps,
      super.replaceCancelButtonWithIcon,
      super.cameraPreviewMode,
      super.touchToFocusEnabled,
      super.screen = "HealthInsuranceCardScanner"});
}
