import 'dart:ui';

import 'common_platform.dart';
import 'package:json_annotation/json_annotation.dart';
import '../sdk_utils.dart';

part 'license_plate_configuration_json.g.dart';

@JsonSerializable(
    includeIfNull: false, createFactory: false, explicitToJson: true)
class LicensePlateScannerJsonConfiguration {
  /// A version of the configuration object
  String? version;

  /// The configuration object should be applied for this screen
  String? screen;

  /// The preferred camera module (default - BACK)
  CameraModule? cameraModule;

  @JsonKey(toJson: toColorString)

  /// Background color outside of the finder window.
  Color? cameraOverlayColor;

  /// Title of the cancel button.
  String? cancelButtonTitle;

  /// Whether the cancel button is hidden or not. iOS only.
  bool? cancelButtonHidden;

  /// The style of the confirmation dialog. iOS Only.
  DialogStyle? confirmationDialogStyle;

  /// The style of the confirmation dialogs confirm button. iOS Only.
  DialogButtonStyle? confirmationDialogConfirmButtonStyle;

  /// The style of the confirmation dialogs retry button. iOS Only.
  DialogButtonStyle? confirmationDialogRetryButtonStyle;

  /// Title of the button that opens the screen where the user can allow the usage of the camera by the app.
  String? enableCameraButtonTitle;

  /// Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera.
  String? enableCameraExplanationText;

  @JsonKey(toJson: toColorString)

  /// Foreground color of the detection overlay.
  Color? finderLineColor;

  /// Width of finder frame border. Default is 2.
  double? finderLineWidth;

  /// User guidance text below the finder view
  String? finderTextHint;

  @JsonKey(toJson: toColorString)

  /// Foreground color of the description label.
  Color? finderTextHintColor;

  /// The preferred height of the view finder. Defaults to 150.0. iOS only.
  double? finderPreferredHeight;

  /// Controls whether the flash toggle button is hidden or not. iOS only.
  bool? flashButtonHidden;

  /// Title of the flash toggle button. iOS only.
  String? flashButtonTitle;

  /// Controls whether the flash should be initially enabled. The default value is FALSE.
  bool? flashEnabled;

  /// UI Interface orientation lock mode
  OrientationLockMode? orientationLockMode;

  @JsonKey(toJson: toColorString)

  /// The background color of the top toolbar.
  Color? topBarBackgroundColor;

  @JsonKey(toJson: toColorString)

  /// The color of all active toggle buttons in the toolbar.
  Color? topBarButtonsActiveColor;

  @JsonKey(toJson: toColorString)

  /// The color of all inactive toggle buttons in the toolbar.
  Color? topBarButtonsInactiveColor;

  /// Minimum number of accumulated frames that have equal result
  int? minNumberOfRequiredFramesWithEqualRecognitionResult;

  /// Maximum number of accumulated frames to inspect before actual result is returned
  int? maximumNumberOfAccumulatedFrames;

  /// Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only.
  bool? useButtonsAllCaps;

  /// Threshold used to pause the detection after significant movement occurred. -1 is default value. Default = 0 for textFilterStrategy='DOCUMENT' and 1000 for textFilterStrategy='LCD_DOT_MATRIX_DISPLAY'. Android only.
  int? significantShakeDelay;

  /// Detector mode, classic (OCR based) or ML (machine learning based) approach.
  LicensePlateScanStrategy? scanStrategy;

  /// The title of the confirmation dialog.
  String? confirmationDialogTitle;

  /// The message text of the confirmation dialog.
  String? confirmationDialogMessage;

  /// The title of the confirmation dialog retry button.
  String? confirmationDialogConfirmButtonTitle;

  /// The title of the confirmation dialog confirm button.
  String? confirmationDialogRetryButtonTitle;

  @JsonKey(toJson: toColorString)

  /// The accent color of buttons on a confirmation dialog. Android only.
  Color? confirmationDialogAccentColor;

  /// Allows to set if the confirm button should be filled. Defaults to TRUE. Android only.
  bool? confirmationDialogConfirmButtonFilled;

  @JsonKey(toJson: toColorString)

  /// Allows to set a text color of the filled button. See `confirmationDialogConfirmButtonFilled`. Android only.
  Color? confirmationDialogConfirmButtonFilledTextColor;

  /// If `true`, replaces the cancel button in the top bar with a back arrow icon. The default value is FALSE. Android only.
  bool? replaceCancelButtonWithIcon;

  /// Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only
  CameraPreviewMode? cameraPreviewMode;

  /// Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only.
  bool? touchToFocusEnabled;

  LicensePlateScannerJsonConfiguration({
    this.version,
    this.screen,
    this.cameraModule,
    this.cameraOverlayColor,
    this.cancelButtonTitle,
    this.cancelButtonHidden,
    this.confirmationDialogStyle,
    this.confirmationDialogConfirmButtonStyle,
    this.confirmationDialogRetryButtonStyle,
    this.enableCameraButtonTitle,
    this.enableCameraExplanationText,
    this.finderLineColor,
    this.finderLineWidth,
    this.finderTextHint,
    this.finderTextHintColor,
    this.finderPreferredHeight,
    this.flashButtonHidden,
    this.flashButtonTitle,
    this.flashEnabled,
    this.orientationLockMode,
    this.topBarBackgroundColor,
    this.topBarButtonsActiveColor,
    this.topBarButtonsInactiveColor,
    this.minNumberOfRequiredFramesWithEqualRecognitionResult,
    this.maximumNumberOfAccumulatedFrames,
    this.useButtonsAllCaps,
    this.significantShakeDelay,
    this.scanStrategy,
    this.confirmationDialogTitle,
    this.confirmationDialogMessage,
    this.confirmationDialogConfirmButtonTitle,
    this.confirmationDialogRetryButtonTitle,
    this.confirmationDialogAccentColor,
    this.confirmationDialogConfirmButtonFilled,
    this.confirmationDialogConfirmButtonFilledTextColor,
    this.replaceCancelButtonWithIcon,
    this.cameraPreviewMode,
    this.touchToFocusEnabled,
  });

  Map<String, dynamic> toJson() =>
      _$LicensePlateScannerJsonConfigurationToJson(this);
}

/// Detector mode, classic (OCR based) or ML (machine learning based) approach.
enum LicensePlateScanStrategy {
  /// OCR based */
  CLASSIC,

  /// Machine learning based */
  ML_BASED,
}
